@@ -20,7 +20,7 @@ import Language.Haskell.GhclibParserEx.GHC.Driver.Session as GhclibParserEx
 import DynFlags hiding (verbosity)
 
 import Language.Preprocessor.Cpphs
-import System.Console.ANSI(hSupportsANSI)
+import System.Console.ANSI(hSupportsANSIWithoutEmulation)
 import System.Console.CmdArgs.Explicit(helpText, HelpFormat(..))
 import System.Console.CmdArgs.Implicit
 import System.Directory.Extra
@@ -29,12 +29,12 @@ import System.Exit
 import System.FilePath
 import System.IO
 import System.IO.Error
-import System.Info.Extra
 import System.Process
 import System.FilePattern
 
 import EmbedData
 import Util
+import Timing
 import Extension
 import Paths_hlint
 import Data.Version
@@ -47,10 +47,7 @@ getCmd args = withArgs (map f args) $ automatic =<< cmdArgsRun mode
 
 
 automatic :: Cmd -> IO Cmd
-automatic cmd = case cmd of
-    CmdMain{} -> dataDir =<< path =<< git =<< extension cmd
-    CmdGrep{} -> path =<< extension cmd
-    CmdTest{} -> dataDir cmd
+automatic cmd = dataDir =<< path =<< git =<< extension cmd
     where
         path cmd = pure $ if null $ cmdPath cmd then cmd{cmdPath=["."]} else cmd
         extension cmd = pure $ if null $ cmdExtension cmd then cmd{cmdExtension=["hs","lhs"]} else cmd
@@ -70,7 +67,8 @@ automatic cmd = case cmd of
                     Just git -> do
                         let args = ["ls-files", "--cached", "--others", "--exclude-standard"] ++
                                    map ("*." ++) (cmdExtension cmd)
-                        files <- readProcess git args ""
+                        files <- timedIO "Execute" (unwords $ git:args) $
+                            readProcess git args ""
                         pure cmd{cmdFiles = cmdFiles cmd ++ lines files}
             | otherwise = pure cmd
 
@@ -90,7 +88,7 @@ data ColorMode
 
 
 instance Default ColorMode where
-  def = if isWindows then Never else Auto
+  def = Auto
 
 
 data Cmd
@@ -127,29 +125,8 @@ data Cmd
         ,cmdRefactorOptions :: String   -- ^ Options to pass to the `refactor` executable.
         ,cmdWithRefactor :: FilePath    -- ^ Path to refactor tool
         ,cmdIgnoreGlob :: [FilePattern]
-        }
-    | CmdGrep
-        {cmdFiles :: [FilePath]    -- ^ which files to run it on, nothing = none given
-        ,cmdPattern :: String
-        ,cmdExtension :: [String]        -- ^ extensions
-        ,cmdLanguage :: [String]      -- ^ the extensions (may be prefixed by "No")
-        ,cmdPath :: [String]
-        ,cmdCppDefine :: [String]
-        ,cmdCppInclude :: [FilePath]
-        ,cmdCppFile :: [FilePath]
-        ,cmdCppSimple :: Bool
-        ,cmdCppAnsi :: Bool
-        }
-    | CmdTest
-        {cmdProof :: [FilePath]          -- ^ a proof script to check against
-        ,cmdGivenHints :: [FilePath]     -- ^ which settings files were explicitly given
-        ,cmdDataDir :: FilePath          -- ^ the data directory
-        ,cmdReports :: [FilePath]        -- ^ where to generate reports
-        ,cmdTempDir :: FilePath          -- ^ temporary directory to put the files in
-        ,cmdQuickCheck :: Bool
-        ,cmdTypeCheck :: Bool
-        ,cmdWithRefactor :: FilePath
-        ,cmdGenerateSummary :: Bool      -- ^ Generate a summary of built-in hints
+        ,cmdGenerateSummary :: [FilePath]  -- ^ Generate a summary of available hints
+        ,cmdTest :: Bool
         }
     deriving (Data,Typeable,Show)
 
@@ -160,7 +137,7 @@ mode = cmdArgsMode $ modes
         ,cmdGivenHints = nam "hint" &= typFile &= help "Hint/ignore file to use"
         ,cmdWithGroups = nam_ "with-group" &= typ "GROUP" &= help "Extra hint groups to use"
         ,cmdGit = nam "git" &= help "Run on files tracked by git"
-        ,cmdColor = nam "colour" &= name "color" &= opt Always &= typ "always/never/auto" &= help "Color output (requires ANSI terminal; auto means on when $TERM is supported; by itself, selects always)"
+        ,cmdColor = nam "colour" &= name "color" &= opt Always &= typ "always/never/auto" &= help "Color output (requires an ANSI terminal; 'auto' means on if the standard output channel can support ANSI; by itself, selects 'always')"
         ,cmdThreads = 1 &= name "threads" &= name "j" &= opt (0 :: Int) &= help "Number of threads to use (-j for all)"
         ,cmdIgnore = nam "ignore" &= typ "HINT" &= help "Ignore a particular hint"
         ,cmdShowAll = nam "show" &= help "Show all ignored ideas"
@@ -187,24 +164,15 @@ mode = cmdArgsMode $ modes
         ,cmdRefactorOptions = nam_ "refactor-options" &= typ "OPTIONS" &= help "Options to pass to the `refactor` executable"
         ,cmdWithRefactor = nam_ "with-refactor" &= help "Give the path to refactor"
         ,cmdIgnoreGlob = nam_ "ignore-glob" &= help "Ignore paths matching glob pattern"
+        ,cmdGenerateSummary = nam_ "generate-summary" &= opt "hints.md" &= help "Generate a summary of built-in hints"
+        ,cmdTest = nam_ "test" &= help "Run the test suite"
         } &= auto &= explicit &= name "lint"
-    ,CmdGrep
-        {cmdFiles = def &= args &= typ "FILE/DIR"
-        ,cmdPattern = def &= argPos 0 &= typ "PATTERN"
-        } &= explicit &= name "grep"
-    ,CmdTest
-        {cmdProof = nam_ "proof" &= typFile &= help "Isabelle/HOLCF theory file"
-        ,cmdTypeCheck = nam_ "typecheck" &= help "Use GHC to type check the hints"
-        ,cmdQuickCheck = nam_ "quickcheck" &= help "Use QuickCheck to check the hints"
-        ,cmdTempDir = nam_ "tempdir" &= help "Where to put temporary files (not cleaned up)"
-        ,cmdGenerateSummary = nam_ "generate-summary" &= help "Generate a summary of built-in hints"
-        } &= explicit &= name "test"
         &= details ["HLint gives hints on how to improve Haskell code."
                    ,""
                    ,"To check all Haskell files in 'src' and generate a report type:"
                    ,"  hlint src --report"]
     ] &= program "hlint" &= verbosity
-    &=  summary ("HLint v" ++ showVersion version ++ ", (C) Neil Mitchell 2006-2020")
+    &=  summary ("HLint v" ++ showVersion version ++ ", (C) Neil Mitchell 2006-2021")
     where
         nam xs = nam_ xs &= name [head xs]
         nam_ xs = def &= explicit &= name xs
@@ -220,7 +188,7 @@ cmdHintFiles cmd = do
         fail $ unlines $ "Failed to find requested hint files:" : map ("  "++) bad
 
     -- if the user has given any explicit hints, ignore the local ones
-    implicit <- if explicit /= [] then pure Nothing else do
+    implicit <- if explicit /= [] || cmdGenerateSummary cmd /= [] then pure Nothing else do
         -- we follow the stylish-haskell config file search policy
         -- 1) current directory or its ancestors; 2) home directory
         curdir <- getCurrentDirectory
@@ -253,7 +221,9 @@ cmdUseColour :: Cmd -> IO Bool
 cmdUseColour cmd = case cmdColor cmd of
   Always -> pure True
   Never  -> pure False
-  Auto   -> hSupportsANSI stdout
+  Auto   -> do
+    supportsANSI <- hSupportsANSIWithoutEmulation stdout
+    pure $ Just True == supportsANSI
 
 
 "." <\> x = x
@@ -286,7 +256,8 @@ getFile _ [] exts _ file = exitMessage $ "Couldn't find file: " ++ file
 getFile ignore (p:ath) exts t file = do
     isDir <- doesDirectoryExist $ p <\> file
     if isDir then do
-        let avoidDir x = let y = takeFileName x in "_" `isPrefixOf` y || ("." `isPrefixOf` y && not (all (== '.') y))
+        let ignoredDirectories = ["dist", "dist-newstyle"]
+            avoidDir x = let y = takeFileName x in "_" `isPrefixOf` y || ("." `isPrefixOf` y && not (all (== '.') y)) || y `elem` ignoredDirectories
             avoidFile x = let y = takeFileName x in "." `isPrefixOf` y || ignore x
         xs <- listFilesInside (pure . not . avoidDir) $ p <\> file
         pure [x | x <- xs, drop1 (takeExtension x) `elem` exts, not $ avoidFile x]
@@ -328,7 +299,7 @@ getExtensions args =
         f (a, e) ('N':'o':x) | Just x <- GhclibParserEx.readExtension x, let xs = expandDisable x =
             (deletes xs a, xs ++ deletes xs e)
         f (a, e) x | Just x <- GhclibParserEx.readExtension x = (x : delete x a, delete x e)
-        f (a, e) x = (a, e) -- Ignore unknown extension.
+        f (a, e) x = error $ "Unknown extension: '" ++ x ++ "'"
 
         deletes [] ys = ys
         deletes (x:xs) ys = deletes xs $ delete x ys
