@@ -33,8 +33,9 @@ import Data.Data
 import Data.Generics.Uniplate.DataOnly
 import Data.List.Extra
 import Data.Tuple.Extra
+import Data.Maybe
 
-import Refact (toSS)
+import Refact (substVars, toSS)
 import Refact.Types hiding (SrcSpan, Match)
 import qualified Refact.Types as R (SrcSpan)
 
@@ -102,8 +103,7 @@ descendBracket op x = descendIndex g x
     where
         g i y = if a then f i b else b
             where (a, b) = op y
-        f i y@(L _ e) | needBracket i x y = addParen y
-        f _ y = y
+        f i y = if needBracket i x y then addParen y else y
 
 -- Add brackets as suggested 'needBracket at 1-level of depth.
 rebracket1 :: LHsExpr GhcPs -> LHsExpr GhcPs
@@ -203,8 +203,8 @@ niceLambdaR [x] y
   where
     -- Factor the expression with respect to x.
     factor :: LHsExpr GhcPs -> Maybe (LHsExpr GhcPs, [LHsExpr GhcPs])
-    factor y@(L _ (HsApp _ ini lst)) | view lst == Var_ x = Just (ini, [ini])
-    factor y@(L _ (HsApp _ ini lst)) | Just (z, ss) <- factor lst
+    factor (L _ (HsApp _ ini lst)) | view lst == Var_ x = Just (ini, [ini])
+    factor (L _ (HsApp _ ini lst)) | Just (z, ss) <- factor lst
       = let r = niceDotApp ini z
         in if astEq r z then Just (r, ss) else Just (r, ini : ss)
     factor (L _ (OpApp _ y op (factor -> Just (z, ss))))| isDol op
@@ -214,7 +214,7 @@ niceLambdaR [x] y
     factor _ = Nothing
     mkRefact :: [LHsExpr GhcPs] -> R.SrcSpan -> Refactoring R.SrcSpan
     mkRefact subts s =
-      let tempSubts = zipWith (\a b -> ([a], toSS b)) ['a' .. 'z'] subts
+      let tempSubts = zipWith (\a b -> (a, toSS b)) substVars subts
           template = dotApps (map (strToVar . fst) tempSubts)
       in Replace Expr s tempSubts (unsafePrettyPrint template)
 -- Rewrite @\x y -> x + y@ as @(+)@.
@@ -228,10 +228,11 @@ niceLambdaR [x, y] (view -> App2 op (view -> Var_ y1) (view -> Var_ x1))
       )
   where
     gen = noLoc . HsApp noExtField (strToVar "flip")
+        . if isAtom op then id else addParen
 
 -- We're done factoring, but have no variables left, so we shouldn't make a lambda.
 -- @\ -> e@ ==> @e@
-niceLambdaR [] e = (e, const [])
+niceLambdaR [] e = (e, \s -> [Replace Expr s [("a", toSS e)] "a"])
 -- Base case. Just a good old fashioned lambda.
 niceLambdaR ss e =
   let grhs = noLoc $ GRHS noExtField [] e :: LGRHS GhcPs (LHsExpr GhcPs)
@@ -307,5 +308,4 @@ descendBracketOld op x = (descendIndex g1 x, descendIndex g2 x)
       _ -> False
 
 fromParen1 :: LHsExpr GhcPs -> LHsExpr GhcPs
-fromParen1 (L _ (HsPar _ x)) = x
-fromParen1 x = x
+fromParen1 x = fromMaybe x $ remParen x
