@@ -5,19 +5,19 @@
 Find bindings within a let, and lists of statements
 If you have n the same, error out
 
-<TEST>
+<TEST_DISABLED_1150>
 foo = a where {a = 1; b = 2; c = 3} \
-bar = a where {a = 1; b = 2; c = 3} -- ??? @NoRefactor
+bar = a where {a = 1; b = 2; c = 3} -- ???
 main = do a; a; a; a
-main = do a; a; a; a; a; a -- ??? @NoRefactor: refactoring not supported for duplication hints.
-main = do a; a; a; a; a; a; a -- ??? @NoRefactor
-main = do (do b; a; a; a); do (do c; a; a; a) -- ??? @NoRefactor
-main = do a; a; a; b; a; a; a -- ??? @NoRefactor
+main = do a; a; a; a; a; a -- ???
+main = do a; a; a; a; a; a; a -- ???
+main = do (do b; a; a; a); do (do c; a; a; a) -- ???
+main = do a; a; a; b; a; a; a -- ???
 main = do a; a; a; b; a; a
-{-# ANN main "HLint: ignore Reduce duplication" #-}; main = do a; a; a; a; a; a -- @Ignore ??? @NoRefactor
-{-# HLINT ignore main "Reduce duplication" #-}; main = do a; a; a; a; a; a -- @Ignore ??? @NoRefactor
-{- HLINT ignore main "Reduce duplication" -}; main = do a; a; a; a; a; a -- @Ignore ??? @NoRefactor
-</TEST>
+{-# ANN main "HLint: ignore Reduce duplication" #-}; main = do a; a; a; a; a; a -- @Ignore ???
+{-# HLINT ignore main "Reduce duplication" #-}; main = do a; a; a; a; a; a -- @Ignore ???
+{- HLINT ignore main "Reduce duplication" -}; main = do a; a; a; a; a; a -- @Ignore ???
+</TEST_DISABLED_1150>
 -}
 
 
