@@ -139,6 +139,8 @@ main = "test"
 {-# LANGUAGE OverloadedStrings #-} \
 main = id --
 {-# LANGUAGE OverloadedLists #-} \
+main = []
+{-# LANGUAGE OverloadedLists #-} \
 main = [1]
 {-# LANGUAGE OverloadedLists #-} \
 main [1] = True
@@ -216,13 +218,23 @@ static = 42 --
 import GHC.TypeLits(KnownNat, type (+), type (*))
 {-# LANGUAGE LambdaCase, MultiWayIf, NoRebindableSyntax #-} \
 foo = \case True -> 3 -- {-# LANGUAGE LambdaCase, NoRebindableSyntax #-}
+{-# LANGUAGE ImportQualifiedPost #-} \
+import Control.Monad qualified as CM
+{-# LANGUAGE ImportQualifiedPost #-} \
+import qualified Control.Monad as CM hiding (mapM) \
+import Data.Foldable -- @NoRefactor: refactor only works when using GHC 8.10
+{-# LANGUAGE StandaloneKindSignatures #-} \
+type T :: (k -> Type) -> k -> Type \
+data T m a = MkT (m a) (T Maybe (m a))
+{-# LANGUAGE NoMonomorphismRestriction, NamedFieldPuns #-} \
+main = 1 -- @Note Extension NamedFieldPuns is not used
 </TEST>
 -}
 
 
 module Hint.Extensions(extensionsHint) where
 
-import Hint.Type(ModuHint, rawIdea,Severity(Warning),Note(..),toSS,ghcAnnotations,ghcModule)
+import Hint.Type(ModuHint,rawIdea,Severity(Warning),Note(..),toSS,ghcAnnotations,ghcModule)
 import Extension
 
 import Data.Generics.Uniplate.DataOnly
@@ -261,8 +273,8 @@ extensionsHint _ x =
         (comment (mkLanguagePragmas sl exts))
         (Just newPragma)
         ( [RequiresExtension (show gone) | (_, Just x) <- before \\ after, gone <- Map.findWithDefault [] x disappear] ++
-            [ Note $ "Extension " ++ show x ++ " is " ++ reason x
-            | (_, Just x) <- explainedRemovals])
+            [ Note $ "Extension " ++ s ++ " is " ++ reason x
+            | (s, Just x) <- explainedRemovals])
         [ModifyComment (toSS (mkLanguagePragmas sl exts)) newPragma]
     | (L sl _,  exts) <- languagePragmas $ pragmas (ghcAnnotations x)
     , let before = [(x, readExtension x) | x <- exts]
@@ -432,6 +444,7 @@ used OverloadedStrings = hasS isString
 used OverloadedLists = hasS isListExpr ||^ hasS isListPat
   where
     isListExpr :: HsExpr GhcPs -> Bool
+    isListExpr (HsVar _ n) = rdrNameStr n == "[]"
     isListExpr ExplicitList{} = True
     isListExpr ArithSeq{} = True
     isListExpr _ = False
@@ -453,6 +466,9 @@ used MagicHash = hasS f ||^ hasS isPrimLiteral
     f :: RdrName -> Bool
     f s = "#" `isSuffixOf` occNameStr s
 used PatternSynonyms = hasS isPatSynBind ||^ hasS isPatSynIE
+used ImportQualifiedPost = hasS (== QualifiedPost)
+used StandaloneKindSignatures = hasT (un :: StandaloneKindSig GhcPs)
+
 used _= const True
 
 hasDerive :: [String] -> Located (HsModule GhcPs) -> Bool
