@@ -28,20 +28,23 @@ f (Foo a b c) = \c -> c + c -- f (Foo a b _) c = c + c
 f a = \x -> x + x where _ = test
 f (test -> a) = \x -> x + x
 f = \x -> x + x -- f x = x + x
-fun x y z = f x y z -- fun = f @NoRefactor: refactoring for eta reduce is not implemented
-fun x y z = f x x y z -- fun x = f x x @NoRefactor
-fun x y z = f g z -- fun x y = f g @NoRefactor
-fun x = f . g $ x -- fun = f . g @NoRefactor
+fun x y z = f x y z -- fun = f
+fun x y z = f x x y z -- fun x = f x x
+fun x y z = f g z -- fun x y = f g
+fun x = f . g $ x -- fun = f . g
 f = foo (\y -> g x . h $ y) -- g x . h
 f = foo (\y -> g x . h $ y) -- @Message Avoid lambda
-f = foo ((*) x) -- (x *) @NoRefactor
+f = foo ((*) x) -- (x *)
+f = foo ((Prelude.*) x) -- (x Prelude.*)
 f = (*) x
-f = foo (flip op x) -- (`op` x) @NoRefactor
-f = foo (flip op x) -- @Message Use section @NoRefactor
+f = foo (flip op x) -- (`op` x)
+f = foo (flip op x) -- @Message Use section
+f = foo (flip x y) -- (`x` y)
 foo x = bar (\ d -> search d table) -- (`search` table)
 foo x = bar (\ d -> search d table) -- @Message Avoid lambda using `infix`
 f = flip op x
-f = foo (flip (*) x) -- (* x) @NoRefactor
+f = foo (flip (*) x) -- (* x)
+f = foo (flip (Prelude.*) x) -- (Prelude.* x)
 f = foo (flip (-) x)
 f = foo (\x y -> fun x y) -- @Warning fun
 f = foo (\x y z -> fun x y z) -- @Warning fun
@@ -52,14 +55,15 @@ f = foo (\x -> x # y)
 f = foo (\x -> \y -> x x y y) -- \x y -> x x y y
 f = foo (\x -> \x -> foo x x) -- \_ x -> foo x x
 f = foo (\(foo -> x) -> \y -> x x y y)
-f = foo (\(x:xs) -> \x -> foo x x) -- \(_:xs) x -> foo x x @NoRefactor
+f = foo (\(x:xs) -> \x -> foo x x) -- \(_:xs) x -> foo x x
 f = foo (\x -> \y -> \z -> x x y y z z) -- \x y z -> x x y y z z
 x ! y = fromJust $ lookup x y
 f = foo (\i -> writeIdea (getClass i) i)
-f = bar (flip Foo.bar x) -- (`Foo.bar` x) @NoRefactor
+f = bar (flip Foo.bar x) -- (`Foo.bar` x)
 f = a b (\x -> c x d)  -- (`c` d)
 yes = \x -> a x where -- a
 yes = \x y -> op y x where -- flip op
+yes = \x y -> op z y x where -- flip (op z)
 f = \y -> nub $ reverse y where -- nub . reverse
 f = \z -> foo $ bar $ baz z where -- foo . bar . baz
 f = \z -> foo $ bar x $ baz z where -- foo . bar x . baz
@@ -69,21 +73,23 @@ f = bar &+& \x -> f (g x)
 foo = [\column -> set column [treeViewColumnTitle := printf "%s (match %d)" name (length candidnates)]]
 foo = [\x -> x]
 foo = [\m x -> insert x x m]
-foo a b c = bar (flux ++ quux) c where flux = a -- foo a b = bar (flux ++ quux) @NoRefactor
+foo a b c = bar (flux ++ quux) c where flux = a -- foo a b = bar (flux ++ quux)
 foo a b c = bar (flux ++ quux) c where flux = c
 yes = foo (\x -> Just x) -- @Warning Just
 foo = bar (\x -> (x `f`)) -- f
 foo = bar (\x -> shakeRoot </> "src" </> x)
-baz = bar (\x -> (x +)) -- (+) @NoRefactor
+baz = bar (\x -> (x +)) -- (+)
 xs `withArgsFrom` args = f args
-foo = bar (\x -> case x of Y z -> z) -- \(Y z) -> z @NoRefactor
-yes = blah (\ x -> case x of A -> a; B -> b) -- \ case A -> a; B -> b @NoRefactor
-yes = blah (\ x -> case x of A -> a; B -> b) -- @Note may require `{-# LANGUAGE LambdaCase #-}` adding to the top of the file @NoRefactor
+foo = bar (\x -> case x of Y z -> z) -- \(Y z) -> z
+foo = bar (\x -> case x of [y, z] -> z) -- \[y, z] -> z
+yes = blah (\ x -> case x of A -> a; B -> b) -- \ case A -> a; B -> b
+yes = blah (\ x -> case x of A -> a; B -> b) -- @Note may require `{-# LANGUAGE LambdaCase #-}` adding to the top of the file
 no = blah (\ x -> case x of A -> a x; B -> b x)
-yes = blah (\ x -> (y, x)) -- (y,) @NoRefactor
-yes = blah (\ x -> (y, x, z+q)) -- (y, , z+q) @NoRefactor
-yes = blah (\ x -> (y, x, y, u, v)) -- (y, , y, u, v) @NoRefactor
-yes = blah (\ x -> (y, x, z+q)) -- @Note may require `{-# LANGUAGE TupleSections #-}` adding to the top of the file @NoRefactor
+foo = bar (\x -> case x of Y z | z > 0 -> z) -- \case Y z | z > 0 -> z
+yes = blah (\ x -> (y, x)) -- (y,)
+yes = blah (\ x -> (y, x, z+q)) -- (y, , z+q)
+yes = blah (\ x -> (y, x, y, u, v)) -- (y, , y, u, v)
+yes = blah (\ x -> (y, x, z+q)) -- @Note may require `{-# LANGUAGE TupleSections #-}` adding to the top of the file
 yes = blah (\ x -> (y, x, z+x))
 tmp = map (\ x -> runST $ action x)
 yes = map (\f -> dataDir </> f) dataFiles -- (dataDir </>)
@@ -99,7 +105,7 @@ f = map (\s -> MkFoo s 0 s) ["a","b","c"]
 
 module Hint.Lambda(lambdaHint) where
 
-import Hint.Type (DeclHint, Idea, Note(RequiresExtension), suggest, warn, toSS, suggestN, ideaNote)
+import Hint.Type (DeclHint, Idea, Note(RequiresExtension), suggest, warn, toSS, suggestN, ideaNote, substVars, toRefactSrcSpan)
 import Util
 import Data.List.Extra
 import Data.Set (Set)
@@ -131,30 +137,36 @@ lambdaDecl
         origBind@FunBind {fun_id = funName@(L loc1 _), fun_matches =
             MG {mg_alts =
                 L _ [L _ (Match _ ctxt@(FunRhs _ Prefix _) pats (GRHSs _ [L _ (GRHS _ [] origBody@(L loc2 _))] bind))]}}))
-    | L _ (EmptyLocalBinds noExtField) <- bind
+    | L _ (EmptyLocalBinds _) <- bind
     , isLambda $ fromParen origBody
     , null (universeBi pats :: [HsExpr GhcPs])
-    = [warn "Redundant lambda" o (gen pats origBody) [Replace Decl (toSS o) subts template]]
-    | length pats2 < length pats, pvars (drop (length pats2) pats) `disjoint` varss bind
-    = [warn "Eta reduce" (reform pats origBody) (reform pats2 bod2)
-          [ -- Disabled, see apply-refact #3
+    = let (newPats, newBody) = fromLambda . lambda pats $ origBody
+          (sub, tpl) = mkSubtsAndTpl newPats newBody
+          gen :: [LPat GhcPs] -> LHsExpr GhcPs -> LHsDecl GhcPs
+          gen ps = uncurry reform . fromLambda . lambda ps
+          refacts = case newBody of
+              -- https://github.com/alanz/ghc-exactprint/issues/97
+              L _ HsCase{} -> []
+              _ -> [Replace Decl (toSS o) sub tpl]
+       in [warn "Redundant lambda" o (gen pats origBody) refacts]
+
+    | let (newPats, newBody) = etaReduce pats origBody
+    , length newPats < length pats, pvars (drop (length newPats) pats) `disjoint` varss bind
+    = let (sub, tpl) = mkSubtsAndTpl newPats newBody
+       in [warn "Eta reduce" (reform pats origBody) (reform newPats newBody)
+            [Replace Decl (toSS $ reform pats origBody) sub tpl]
           ]
-      ]
     where reform :: [LPat GhcPs] -> LHsExpr GhcPs -> LHsDecl GhcPs
-          reform ps b = L loc $ ValD noExtField $
+          reform ps b = L (combineSrcSpans loc1 loc2) $ ValD noExtField $
             origBind
               {fun_matches = MG noExtField (noLoc [noLoc $ Match noExtField ctxt ps $ GRHSs noExtField [noLoc $ GRHS noExtField [] b] $ noLoc $ EmptyLocalBinds noExtField]) Generated}
 
-          loc = combineSrcSpans loc1 loc2
+          mkSubtsAndTpl newPats newBody = (sub, tpl)
+            where
+              (origPats, vars) = mkOrigPats (Just (rdrNameStr funName)) newPats
+              sub = ("body", toSS newBody) : zip vars (map toSS newPats)
+              tpl = unsafePrettyPrint (reform origPats varBody)
 
-          gen :: [LPat GhcPs] -> LHsExpr GhcPs -> LHsDecl GhcPs
-          gen ps = uncurry reform . fromLambda . lambda ps
-
-          (finalpats, body) = fromLambda . lambda pats $ origBody
-          (pats2, bod2) = etaReduce pats origBody
-          (origPats, subtsVars) = mkOrigPats (Just (rdrNameStr funName)) finalpats
-          subts = ("body", toSS body) : zipWith (\x y -> ([x],y)) subtsVars (map toSS finalpats)
-          template = unsafePrettyPrint (reform origPats varBody)
 lambdaDecl _ = []
 
 
@@ -167,18 +179,29 @@ etaReduce (unsnoc -> Just (ps, view -> PVar_ p)) (L _ (HsApp _ x (view -> Var_ y
 etaReduce ps (L loc (OpApp _ x (isDol -> True) y)) = etaReduce ps (L loc (HsApp noExtField x y))
 etaReduce ps x = (ps, x)
 
---Section refactoring is not currently implemented.
 lambdaExp :: Maybe (LHsExpr GhcPs) -> LHsExpr GhcPs -> [Idea]
-lambdaExp _ o@(L _ (HsPar _ (L _ (HsApp _ oper@(L _ (HsVar _ (L _ (rdrNameOcc -> f)))) y))))
+lambdaExp _ o@(L _ (HsPar _ (L _ (HsApp _ oper@(L _ (HsVar _ origf@(L _ (rdrNameOcc -> f)))) y))))
     | isSymOcc f -- is this an operator?
     , isAtom y
     , allowLeftSection $ occNameString f
-    , not $ isTypeApp y =
-      [suggestN "Use section" o $ noLoc $ HsPar noExtField $ noLoc $ SectionL noExtField y oper]
+    , not $ isTypeApp y
+    = [suggest "Use section" o to [r]]
+    where
+        to :: LHsExpr GhcPs
+        to = noLoc $ HsPar noExtField $ noLoc $ SectionL noExtField y oper
+        r = Replace Expr (toSS o) [("x", toSS y)] ("(x " ++ unsafePrettyPrint origf ++ ")")
 
-lambdaExp _ o@(L _ (HsPar _ (view -> App2 (view -> Var_ "flip") origf@(view -> Var_ f) y)))
-    | allowRightSection f, not $ "(" `isPrefixOf` f
-    = [suggestN "Use section" o $ noLoc $ HsPar noExtField $ noLoc $ SectionR noExtField origf y]
+lambdaExp _ o@(L _ (HsPar _ (view -> App2 (view -> Var_ "flip") origf@(view -> RdrName_ f) y)))
+    | allowRightSection (rdrNameStr f), not $ "(" `isPrefixOf` rdrNameStr f
+    = [suggest "Use section" o to [r]]
+    where
+        to :: LHsExpr GhcPs
+        to = noLoc $ HsPar noExtField $ noLoc $ SectionR noExtField origf y
+        op = if isSymbolRdrName (unLoc f)
+               then unsafePrettyPrint f
+               else "`" ++ unsafePrettyPrint f ++ "`"
+        var = if rdrNameStr f == "x" then "y" else "x"
+        r = Replace Expr (toSS o) [(var, toSS y)] ("(" ++ op ++ " " ++ var ++ ")")
 lambdaExp p o@(L _ HsLam{})
     | not $ any isOpApp p
     , (res, refact) <- niceLambdaR [] o
@@ -187,8 +210,11 @@ lambdaExp p o@(L _ HsLam{})
     , not $ "runST" `Set.member` Set.map occNameString (freeVars o)
     , let name = "Avoid lambda" ++ (if countRightSections res > countRightSections o then " using `infix`" else "")
     -- If the lambda's parent is an HsPar, and the result is also an HsPar, the span should include the parentheses.
-    , let from = case (p, res) of
-              (Just p@(L _ (HsPar _ (L _ HsLam{}))), L _ HsPar{}) -> p
+    , let from = case p of
+              -- Avoid creating redundant bracket.
+              Just p@(L _ (HsPar _ (L _ HsLam{})))
+                | L _ HsPar{} <- res -> p
+                | L _ (HsVar _ (L _ name)) <- res, not (isSymbolRdrName name) -> p
               _ -> o
     = [(if isVar res then warn else suggest) name from res (refact $ toSS from)]
     where
@@ -202,8 +228,8 @@ lambdaExp p o@(SimpleLambda origPats origBody)
     [suggest "Collapse lambdas" o (lambda pats body) [Replace Expr (toSS o) subts template]]
     where
       (pats, body) = fromLambda o
-      (oPats, subtsVars) = mkOrigPats Nothing pats
-      subts = ("body", toSS body) : zipWith (\x y -> ([x],y)) subtsVars (map toSS pats)
+      (oPats, vars) = mkOrigPats Nothing pats
+      subts = ("body", toSS body) : zip vars (map toSS pats)
       template = unsafePrettyPrint (lambda oPats varBody)
 
 -- match a lambda with a variable pattern, with no guards and no where clauses
@@ -230,18 +256,35 @@ lambdaExp _ o@(SimpleLambda [view -> PVar_ x] (L _ expr)) =
                  -- we need to
                  --     * add brackets to the match, because matches in lambdas require them
                  --     * mark match as being in a lambda context so that it's printed properly
-                 oldMG@(MG _ (L _ [L _ oldmatch]) _) ->
-                     [suggestN "Use lambda" o $ noLoc $ HsLam noExtField oldMG
-                         { mg_alts = noLoc
-                             [noLoc oldmatch
-                                 { m_pats = map mkParPat $ m_pats oldmatch
-                                 , m_ctxt = LambdaExpr
+                 oldMG@(MG _ (L _ [L _ oldmatch]) _)
+                   | all (\(L _ (GRHS _ stmts _)) -> null stmts) (grhssGRHSs (m_grhss oldmatch)) ->
+                     let patLocs = fmap getLoc (m_pats oldmatch)
+                         bodyLocs = concatMap (\case L _ (GRHS _ _ body) -> [getLoc body]; _ -> [])
+                                        $ grhssGRHSs (m_grhss oldmatch)
+                         r | notNull patLocs && notNull bodyLocs =
+                             let xloc = foldl1' combineSrcSpans patLocs
+                                 yloc = foldl1' combineSrcSpans bodyLocs
+                              in [ Replace Expr (toSS o) [("x", toRefactSrcSpan xloc), ("y", toRefactSrcSpan yloc)]
+                                     ((if needParens then "\\(x)" else "\\x") ++ " -> y")
+                                 ]
+                           | otherwise = []
+                         needParens = any (patNeedsParens appPrec . unLoc) (m_pats oldmatch)
+                      in [ suggest "Use lambda" o
+                             ( noLoc $ HsLam noExtField oldMG
+                                 { mg_alts = noLoc
+                                     [ noLoc oldmatch
+                                         { m_pats = map mkParPat $ m_pats oldmatch
+                                         , m_ctxt = LambdaExpr
+                                         }
+                                     ]
                                  }
-                             ] }
-                     ]
+                               :: LHsExpr GhcPs
+                             )
+                             r
+                         ]
 
                  -- otherwise we should use @LambdaCase@
-                 MG _ (L _ xs) _ ->
+                 MG _ (L _ _) _ ->
                      [(suggestN "Use lambda-case" o $ noLoc $ HsLamCase noExtField matchGroup)
                          {ideaNote=[RequiresExtension "LambdaCase"]}]
                  _ -> []
@@ -250,7 +293,7 @@ lambdaExp _ o@(SimpleLambda [view -> PVar_ x] (L _ expr)) =
         -- | Filter out tuple arguments, converting the @x@ (matched in the lambda) variable argument
         -- to a missing argument, so that we get the proper section.
         removeX :: LHsTupArg GhcPs -> LHsTupArg GhcPs
-        removeX arg@(L _ (Present _ (view -> Var_ x')))
+        removeX (L _ (Present _ (view -> Var_ x')))
             | x == x' = noLoc $ Missing noExtField
         removeX y = y
         -- | Extract the name of an argument of a tuple if it's present and a variable.
@@ -274,22 +317,22 @@ fromLambda x = ([], x)
 
 -- | For each pattern, if it does not contain wildcards, replace it with a variable pattern.
 --
--- The second component of the result is a list of substitution variables, which is ['a'..'z'],
--- excluding variables that occur in the function name or patterns with wildcards. For example, given
--- 'f (Foo a b _) = ...', 'f', 'a' and 'b' are removed.
-mkOrigPats :: Maybe String -> [LPat GhcPs] -> ([LPat GhcPs], [Char])
-mkOrigPats funName pats = (zipWith munge subtsVars pats', subtsVars)
+-- The second component of the result is a list of substitution variables, which are guaranteed
+-- to not occur in the function name or patterns with wildcards. For example, given
+-- 'f (Foo a b _) = ...', 'f', 'a' and 'b' are not usable as substitution variables.
+mkOrigPats :: Maybe String -> [LPat GhcPs] -> ([LPat GhcPs], [String])
+mkOrigPats funName pats = (zipWith munge vars pats', vars)
   where
     (Set.unions -> used, pats') = unzip (map f pats)
 
     -- Remove variables that occur in the function name or patterns with wildcards
-    subtsVars = filter (\c -> c `Set.notMember` used && Just [c] /= funName) ['a'..'z']
+    vars = filter (\s -> s `Set.notMember` used && Just s /= funName) substVars
 
     -- Returns (chars in the pattern if the pattern contains wildcards, (whether the pattern contains wildcards, the pattern))
-    f :: LPat GhcPs -> (Set Char, (Bool, LPat GhcPs))
+    f :: LPat GhcPs -> (Set String, (Bool, LPat GhcPs))
     f p
       | any isWildPat (universe p) =
-          let used = Set.fromList [c | (L _ (VarPat _ (rdrNameStr -> [c]))) <- universe p]
+          let used = Set.fromList [rdrNameStr name | (L _ (VarPat _ name)) <- universe p]
            in (used, (True, p))
       | otherwise = (mempty, (False, p))
 
@@ -297,6 +340,6 @@ mkOrigPats funName pats = (zipWith munge subtsVars pats', subtsVars)
     isWildPat = \case (L _ (WildPat _)) -> True; _ -> False
 
     -- Replace the pattern with a variable pattern if the pattern doesn't contain wildcards.
-    munge :: Char -> (Bool, LPat GhcPs) -> LPat GhcPs
+    munge :: String -> (Bool, LPat GhcPs) -> LPat GhcPs
     munge _ (True, p) = p
-    munge ident (False, L ploc _) = L ploc (VarPat noExtField (L ploc $ mkRdrUnqual $ mkVarOcc [ident]))
+    munge ident (False, L ploc _) = L ploc (VarPat noExtField (L ploc $ mkRdrUnqual $ mkVarOcc ident))
