@@ -4,12 +4,11 @@ module Idea(
     Idea(..),
     rawIdea, idea, suggest, suggestRemove, ideaRemove, warn, ignore,
     rawIdeaN, suggestN, ignoreNoSuggestion,
-    showIdeasJson, showANSI,
+    showIdeasJson, showIdeaANSI,
     Note(..), showNotes,
     Severity(..),
     ) where
 
-import Data.Functor
 import Data.List.Extra
 import Config.Type
 import HsColour
@@ -60,6 +59,7 @@ showIdeaJson idea@Idea{ideaSpan=srcSpan@SrcSpan{..}, ..} = dict
     dict xs = "{" ++ intercalate "," [show k ++ ":" ++ v | (k,v) <- xs] ++ "}"
     list xs = "[" ++ intercalate "," xs ++ "]"
 
+-- | Show a list of 'Idea' values as a JSON string.
 showIdeasJson :: [Idea] -> String
 showIdeasJson ideas = "[" ++ intercalate "\n," (map showIdeaJson ideas) ++ "]"
 
@@ -67,8 +67,9 @@ instance Show Idea where
     show = showEx id
 
 
-showANSI :: IO (Idea -> String)
-showANSI = showEx <$> hsColourConsole
+-- | Show an 'Idea' with ANSI color codes to give syntax coloring to the Haskell code.
+showIdeaANSI :: Idea -> String
+showIdeaANSI = showEx hsColourConsole
 
 showEx :: (String -> String) -> Idea -> String
 showEx tt Idea{..} = unlines $
